Package: BiocBuildReporter
Type: Package
Title: Functions to process a bioconductor build report database
Version: 1.1.1
Description: This package reads remote parquet files that have processed Bioconductor build report logs. Users may query the tables directly for specific information or use pre-defined helper functions for common queries. The logs processed are from https://bioconductor.org/checkResults/. In the future we will extend this package out to include processing of r-universe logs.
Depends: R (>= 4.6.0)
Imports: arrow, dplyr, BiocFileCache
License: Apache License (>= 2)
Encoding: UTF-8
RoxygenNote: 7.3.3
Authors@R: c(
    person(given="Sean", family="Davis", role = c("aut"), email = 'seandavi@gmail.com'),
    person(given="Lori", family="Shepherd", role = c("aut", "cre"), email = "lori.shepherd@roswellpark.org",comment=c(ORCID = "0000-0002-3242-0582")))    
Suggests: BiocStyle, testthat (>= 3.0.0), knitr, rmarkdown, ggplot2,
        tidyr, stringr
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://github.com/lshep/BiocBuildReporter.git
BugReports: https://github.com/lshep/BiocBuildReporter/issues
biocViews: Software, Infrastructure
git_url: https://git.bioconductor.org/packages/BiocBuildReporter
git_branch: devel
git_last_commit: 13ae3c4
git_last_commit_date: 2026-05-20
Repository: Bioconductor 3.24
Date/Publication: 2026-06-01
NeedsCompilation: no
Packaged: 2026-06-01 21:31:56 UTC; biocbuild
Author: Sean Davis [aut],
  Lori Shepherd [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3242-0582>)
Maintainer: Lori Shepherd <lori.shepherd@roswellpark.org>
