utils::globalVariables(
    c("Package", "Version", "endedat", "episode", "first_failure",
      "gap_days", "gap_hours", "git_branch", "git_last_commit",
      "git_last_commit_date", "last_modified", "node", "old_last_modified",
      "package", "propagate", "stage", "startedat", "status", "version_obj")
    )
